<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
//use OhMyBrew\ShopifyApp\Facades\ShopifyApp;
//use Illuminate\Support\Facades\Auth;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
//use App\Model\Shopify\Product;
use Log;

class HomeController extends Controller {

    private $shApi;
    private $shOptions;
    private $productObj;

    public function __construct() {

      // Create options for the API
      $this->shOptions = new Options();
      $this->shOptions->setVersion(env('SHOPIFY_STORE_API_VERSION', true));

      // Create the client and session
      $this->shApi = new BasicShopifyAPI($this->shOptions);
      $this->shApi->setSession(new Session(env('SHOPIFY_STORE_DOMAIN', true), env('SHOPIFY_STORE_PASSWORD', true)));
    }

    public function loadVariants() {

      //$shop    = Auth::user();
      $limit = 20;
      $previousLink = '';
      $nextLink = '';
      $getPageLink = '';

      if(!isset($_GET['page'])) {
        //$data = $shop->api()->rest('GET', '/admin/products.json',["published_status" => "published", "limit" => $limit]);
        $data = $this->shApi->rest('GET', '/admin/products.json',["published_status" => "published", "limit" => $limit]);
        $data = (object)$data;
        $previousLink = !empty($data->link->previous)? $previousLink = $data->link->previous:'';
        $nextLink = !empty($data->link->next)? $nextLink = $data->link->next:'';
      } else {
        $getPageLink = trim($_GET['page']);
        // $data = $shop->api()->rest('GET', '/admin/products.json',["limit" => $limit,
        // 'page_info' => $getPageLink]);
        $data = $this->shApi->rest('GET', '/admin/products.json',["limit" => $limit,
        'page_info' => $getPageLink]);
        $data = (object)$data;
        $previousLink = !empty($data->link->previous)? $previousLink = $data->link->previous:'';
        $nextLink = !empty($data->link->next)? $nextLink = $data->link->next:'';
      }

      // echo '<pre>';
      // print_r($data->body->products); die;
      $productData = [];
      foreach($data->body->products as $product) {
        foreach($product->variants as $variant) {
          $title = $product->title;
          if($variant->title != "Default Title" && count($product->variants)>1) {
            $title = $title." ".$variant->title;
          }
          $productData[] = [ "sku" => $variant->sku, "product_id" => $product->id,
          "variant_id" => $variant->id, "title" => $title ];
        }
      }

      return view('variants',["products" => $productData, "previousLink" => $previousLink,
      "nextLink" => $nextLink]);

    }

    public function loadProducts() {

        //$shop    = Auth::user();
        $limit = 20;
        $previousLink = '';
        $nextLink = '';
        $getPageLink = '';

        if(!isset($_GET['page'])) {
          //$data = $shop->api()->rest('GET', '/admin/products.json',["published_status" => "published","limit" => $limit]);
          $data = $this->shApi->rest('GET', '/admin/products.json',["published_status" => "published","limit" => $limit]);
          $data = (object)$data;
          $previousLink = !empty($data->link->previous)? $previousLink = $data->link->previous:'';
          $nextLink = !empty($data->link->next) ? $data->link->next : '';
        } else {
          $getPageLink = trim($_GET['page']);
          // $data = $shop->api()->rest('GET', '/admin/products.json',["limit" => $limit,
          // 'page_info' => $getPageLink]);
          $data = $this->shApi->rest('GET', '/admin/products.json',["limit" => $limit,
          'page_info' => $getPageLink]);
          $data = (object)$data;
          $previousLink = !empty($data->link->previous)? $previousLink = $data->link->previous:'';
          $nextLink = !empty($data->link->next) ? $data->link->next : '';
        }

        return view('welcome',["products" => $data->body->products, "previousLink" => $previousLink,
        "nextLink" => $nextLink]);
    }

    public function separatePageToken($url) {

      if (!empty($url)) {
        $url_components = parse_url($url);
        parse_str($url_components['query'], $params);
        $nextPageToken = $params['page_info'];
        return $nextPageToken;
      }
      return '';
    }

    public function search() {

        //$shop    = Auth::user();
        $limit = 10;
        $title = $_POST['search_key'];
        $previousLink = '';
        $nextLink = '';
        $searchedProducts = [];


        do {
          if(empty($previousLink) && empty($nextLink)) {

              // $productDetails = $shop->api()->rest('GET',
              // '/admin/products.json',["limit" => $limit, 
              // "published_status" => "published", "fields" => "id,title,handle"]);
              $productDetails = $this->shApi->rest('GET',
              '/admin/products.json',["limit" => $limit, 
              "published_status" => "published", "fields" => "id,title,handle"]);
              $productDetails = (object)$productDetails;
              $previousLink = !empty($productDetails->link->previous)? 
              $previousLink = $productDetails->link->previous:'';
              $nextLink = !empty($productDetails->link->next)? 
              $nextLink = $productDetails->link->next:'';

          }else {

              // $productDetails = $shop->api()->rest('GET',
              // '/admin/products.json',["limit" => $limit,
              // "fields" => 'id,title,handle', "page_info" => $nextLink]);
              $productDetails = $this->shApi->rest('GET',
              '/admin/products.json',["limit" => $limit,
              "fields" => 'id,title,handle', "page_info" => $nextLink]);
              $productDetails = (object)$productDetails;
              $previousLink = !empty($productDetails->link->previous)? 
              $previousLink = $productDetails->link->previous:'';
              $nextLink = !empty($productDetails->link->next)? 
              $nextLink = $productDetails->link->next:'';
          }

          $productDetails = $productDetails->body->products;

          if (isset($productDetails)) {

            foreach ($productDetails as $products) {
              if (!empty($products->title)) {

                $productTitle = trim(strtolower(preg_replace('/[^a-zA-Z0-9-_\.]/','-', $products->title)));
                $searchTitle = trim(strtolower(preg_replace('/[^a-zA-Z0-9-_\.]/','-', $title)));
                
                if(!isset($searchTitle) || empty($searchTitle)) {
                  echo "<span class='empty_data'>Search key not found</span";
                  exit(0);
                } else if (strpos($productTitle, $searchTitle) !== false || $productTitle == $searchTitle) {

                  $searchedProducts[] = $products;
                }

              }
            }
          }
          $count = count($productDetails);
          if(empty($nextLink)) {
            break;
          }
        } while($limit == $count);

        

        if(!isset($searchedProducts) || empty($searchedProducts)) {
            echo "<span class='empty_data'>Search key not found</span";
            exit(0);
        }
        $html= '<div class="Polaris-Card">
          <div class="">
            <div class="Polaris-DataTable__Navigation"><button type="button" class="Polaris-Button Polaris-Button--disabled Polaris-Button--plain Polaris-Button--iconOnly" disabled="" aria-label="Scroll table left one column"><span class="Polaris-Button__Content"><span class="Polaris-Button__Icon"><span class="Polaris-Icon"><svg viewBox="0 0 20 20" class="Polaris-Icon__Svg" focusable="false" aria-hidden="true">
                        <path d="M12 16a.997.997 0 0 1-.707-.293l-5-5a.999.999 0 0 1 0-1.414l5-5a.999.999 0 1 1 1.414 1.414L8.414 10l4.293 4.293A.999.999 0 0 1 12 16" fill-rule="evenodd"></path>
                      </svg></span></span></span></button><button type="button" class="Polaris-Button Polaris-Button--plain Polaris-Button--iconOnly" aria-label="Scroll table right one column"><span class="Polaris-Button__Content"><span class="Polaris-Button__Icon"><span class="Polaris-Icon"><svg viewBox="0 0 20 20" class="Polaris-Icon__Svg" focusable="false" aria-hidden="true">
                        <path d="M8 16a.999.999 0 0 1-.707-1.707L11.586 10 7.293 5.707a.999.999 0 1 1 1.414-1.414l5 5a.999.999 0 0 1 0 1.414l-5 5A.997.997 0 0 1 8 16" fill-rule="evenodd"></path>
                      </svg></span></span></span></button></div>
            <div class="Polaris-DataTable">
              <div class="Polaris-DataTable__ScrollContainer">
                <table class="Polaris-DataTable__Table">
                  <thead>
                    <tr>
                      <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--firstColumn Polaris-DataTable__Cell--header" scope="col"></th>
                      <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--firstColumn Polaris-DataTable__Cell--header" scope="col">Product ID</th>
                      <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--header Polaris-DataTable__Cell--firstColumn" scope="col">Title</th>
                      <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--header Polaris-DataTable__Cell--firstColumn" scope="col">Handle</th>
                    </tr>

                  </thead>
                  <tbody>';
                    foreach( $searchedProducts as $product) {
                    $html = $html.  '<tr class="Polaris-DataTable__TableRow">
                        <td scope="row"><a style="padding-left:10px;" class="addnewData" product_id="'.$product->id.'" href="#0" class="addnewData"><i title="expand" class="fa fa-plus"></i></a></td>
                        <td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--firstColumn" scope="row">'.$product->id.'</td>
                        <td class="Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell Polaris-DataTable__Cell--firstColumn">'.$product->title.'</td>
                        <td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--firstColumn" >'.$product->handle.'</td>

                        </tr>';
                    }
                  $html = $html.'</tbody>
                </table>
              </div>
            </div>
          </div>
        </div>';
        echo $html;
    }

    public function searchVariants() {

      //$shop    = Auth::user();
      $limit = 10;
      $title = $_POST['search_key'];
      $previousLink = '';
      $nextLink = '';
      $searchedVariantProducts = [];

      do {
        if(empty($previousLink) && empty($nextLink)) {

          // $productDetails = $shop->api()->rest('GET',
          // '/admin/products.json',["published_status" => "published", "limit" => $limit, 
          // "fields" => "id,title,handle,variants"]);
          $productDetails = $this->shApi->rest('GET',
          '/admin/products.json',["published_status" => "published", "limit" => $limit, 
          "fields" => "id,title,handle,variants"]);
          $productDetails = (object)$productDetails;
          $previousLink = !empty($productDetails->link->previous)? 
          $previousLink = $productDetails->link->previous:'';
          $nextLink = !empty($productDetails->link->next)? 
          $nextLink = $productDetails->link->next:'';

        }else {

            // $productDetails = $shop->api()->rest('GET',
            // '/admin/products.json',["limit" => $limit,
            // "fields" => 'id,title,handle,variants', "page_info" => $nextLink]);
            $productDetails = $this->shApi->rest('GET',
            '/admin/products.json',["limit" => $limit,
            "fields" => 'id,title,handle,variants', "page_info" => $nextLink]);
            $productDetails = (object)$productDetails;
            $previousLink = !empty($productDetails->link->previous)? 
            $previousLink = $productDetails->link->previous:'';
            $nextLink = !empty($productDetails->link->next)? 
            $nextLink = $productDetails->link->next:'';
        }

        $productDetails = $productDetails->body->products;

        if (isset($productDetails)) {

          foreach ($productDetails as $products) {

            foreach($products->variants as $variant) {

              if (trim(strtolower($variant->sku)) == trim(strtolower($title))) {

                $variantTitle = str_replace(" ", "-", $variant->title);

                $searchedVariantProducts[] = array(
                  "productID" => $products->id,
                  "variantID" => $variant->id,
                  "variantSKU" => $variant->sku,
                  "title" => trim(strtolower($variantTitle)) == 'default-title' ?
                  $products->title : $products->title." ".$variant->title
                );
              }
            }
          }
        }
        $count = count($productDetails);
        if(empty($nextLink)) {
          break;
        }
      } while($limit == $count);

      if(!isset($searchedVariantProducts) || empty($searchedVariantProducts))
      {
          echo "<span class='empty_data'>Search key not found</span";
          exit(0);
      }
      $html= '<div class="Polaris-Card">
        <div class="">
          <div class="Polaris-DataTable__Navigation"><button type="button" class="Polaris-Button Polaris-Button--disabled Polaris-Button--plain Polaris-Button--iconOnly" disabled="" aria-label="Scroll table left one column"><span class="Polaris-Button__Content"><span class="Polaris-Button__Icon"><span class="Polaris-Icon"><svg viewBox="0 0 20 20" class="Polaris-Icon__Svg" focusable="false" aria-hidden="true">
                      <path d="M12 16a.997.997 0 0 1-.707-.293l-5-5a.999.999 0 0 1 0-1.414l5-5a.999.999 0 1 1 1.414 1.414L8.414 10l4.293 4.293A.999.999 0 0 1 12 16" fill-rule="evenodd"></path>
                    </svg></span></span></span></button><button type="button" class="Polaris-Button Polaris-Button--plain Polaris-Button--iconOnly" aria-label="Scroll table right one column"><span class="Polaris-Button__Content"><span class="Polaris-Button__Icon"><span class="Polaris-Icon"><svg viewBox="0 0 20 20" class="Polaris-Icon__Svg" focusable="false" aria-hidden="true">
                      <path d="M8 16a.999.999 0 0 1-.707-1.707L11.586 10 7.293 5.707a.999.999 0 1 1 1.414-1.414l5 5a.999.999 0 0 1 0 1.414l-5 5A.997.997 0 0 1 8 16" fill-rule="evenodd"></path>
                    </svg></span></span></span></button></div>
          <div class="Polaris-DataTable">
            <div class="Polaris-DataTable__ScrollContainer">
              <table class="Polaris-DataTable__Table">
                <thead>
                  <tr>
                    <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--firstColumn Polaris-DataTable__Cell--header" scope="col"></th>
                    <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--firstColumn Polaris-DataTable__Cell--header" scope="col">SKU</th>
                    <th data-polaris-header-cell="true" class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--header Polaris-DataTable__Cell--firstColumn" scope="col">Title</th>
                  </tr>

                </thead>
                <tbody>';
                  foreach( $searchedVariantProducts as $product) {

                      $html = $html.  '<tr class="Polaris-DataTable__TableRow">
                      <td scope="row"><a style="padding-left:10px;" class="addnewDataVariant" product_id="'.$product['productID'].'" variant_id="'.$product['variantID'].'" href="#0" ><i title="expand" class="fa fa-plus"></i></a></td>
                      <td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--firstColumn" scope="row">'.$product['variantSKU'].'</td>
                      <td class="Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell Polaris-DataTable__Cell--firstColumn">'.$product['title'].'</td>

                      </tr>';

                    }

                $html = $html.'</tbody>
              </table>
            </div>
          </div>
        </div>
      </div>';
      echo $html;
    }
}
